@@ -33,6 +33,7 @@
 #include "XrdCl/XrdClRedirectorRegistry.hh"
 #include "XrdSys/XrdSysPthread.hh"
 
+#include <stdio.h>
 #include <iostream>
 #include <iomanip>
 #include <limits>
@@ -250,7 +251,7 @@ class ProgressDisplay: public XrdCl::CopyProgressHandler
       std::cerr << checkSum.substr( 0, i+1 ) << " ";
       std::cerr << checkSum.substr( i+1, checkSum.length()-i ) << " ";
 
-      if( url->GetProtocol() == "file" )
+      if( url->IsLocalFile() )
         std::cerr << url->GetPath() << " ";
       else
       {
@@ -431,6 +432,8 @@ XrdCl::XRootDStatus GetDirList( XrdCl::FileSystem        *fs,
     }
   }
 
+  delete list;
+
   return XRootDStatus();
 }
 
@@ -443,35 +446,30 @@ XrdCpFile *IndexRemote( XrdCl::FileSystem *fs,
 {
   using namespace XrdCl;
 
-  XrdCpFile   start;
-  XrdCpFile  *end   = &start;
-  XrdCpFile  *current;
-  URL         source( basePath );
-  int         badUrl;
-
-  std::vector<std::string> *files       = new std::vector<std::string>();
-  std::vector<std::string> *directories = new std::vector<std::string>();
-
   Log *log = DefaultEnv::GetLog();
   log->Debug( AppMsg, "Indexing %s", basePath.c_str() );
 
-  XRootDStatus status = GetDirList( fs, source, files, directories );
-  if( !status.IsOK() )
+  DirectoryList *dirList = 0;
+  XRootDStatus st = fs->DirList( basePath, DirListFlags::Recursive, dirList );
+  if( !st.IsOK() )
   {
     log->Info( AppMsg, "Failed to get directory listing for %s: %s",
-                       source.GetURL().c_str(),
-                       status.GetErrorMessage().c_str() );
+                       basePath.c_str(),
+                       st.GetErrorMessage().c_str() );
+    return 0;
   }
 
-  std::vector<std::string>::iterator it;
-  for( it = files->begin(); it != files->end(); ++it )
+  XrdCpFile start, *current = 0;
+  XrdCpFile *end   = &start;
+  int       badUrl = 0;
+  for( auto itr = dirList->Begin(); itr != dirList->End(); ++itr )
   {
-    std::string file = basePath + "/" + (*it);
-    log->Dump( AppMsg, "Found file %s", file.c_str() );
-
-    current = new XrdCpFile( file.c_str(), badUrl );
+    DirectoryList::ListEntry *e = *itr;
+    std::string path = basePath + '/' + e->GetName();
+    current = new XrdCpFile( path.c_str(), badUrl );
     if( badUrl )
     {
+      // TODO release memory
       log->Error( AppMsg, "Bad URL: %s", current->Path );
       return 0;
     }
@@ -481,17 +479,8 @@ XrdCpFile *IndexRemote( XrdCl::FileSystem *fs,
     end           = current;
   }
 
-  for( it = directories->begin(); it != directories->end(); ++it )
-  {
-    std::string directory = basePath + "/" + (*it);
-    log->Dump( AppMsg, "Found directory %s", directory.c_str() );
+  delete dirList;
 
-    end->Next = IndexRemote( fs, directory, dirOffset );
-    while( end->Next ) end = end->Next;
-  }
-
-  delete files;
-  delete directories;
   return start.Next;
 }
 
@@ -653,7 +642,23 @@ int main( int argc, char **argv )
   std::string dest;
   if( config.dstFile->Protocol == XrdCpFile::isDir ||
       config.dstFile->Protocol == XrdCpFile::isFile )
+  {
     dest = "file://";
+
+    // if it is not an absolute path append cwd
+    if( config.dstFile->Path[0] != '/' )
+    {
+      char buf[FILENAME_MAX];
+      char *cwd = getcwd( buf, FILENAME_MAX );
+      if( !cwd )
+      {
+        std::cerr <<  strerror( errno ) << std::endl;
+        return errno;
+      }
+      dest += cwd;
+      dest += '/';
+    }
+  }
   dest += config.dstFile->Path;
 
   //----------------------------------------------------------------------------
@@ -740,7 +745,22 @@ int main( int argc, char **argv )
     PropertyList *results = new PropertyList;
     std::string source = sourceFile->Path;
     if( sourceFile->Protocol == XrdCpFile::isFile )
-      source = "file://" + source;
+    {
+      // make sure it is an absolute path
+      if( source[0] == '/' )
+        source = "file://" + source;
+      else
+      {
+        char buf[FILENAME_MAX];
+        char *cwd = getcwd( buf, FILENAME_MAX );
+        if( !cwd )
+        {
+          std::cerr <<  strerror( errno ) << std::endl;
+          return errno;
+        }
+        source = "file://" + std::string( cwd ) + '/' + source;
+      }
+    }
 
     AppendCGI( source, config.srcOpq );
 
