@@ -46,6 +46,7 @@
 #include <openssl/ssl.h>
 #include <vector>
 #include <arpa/inet.h>
+#include <sstream>
 #include <ctype.h>
 
 #define XRHTTP_TK_GRACETIME     600
@@ -104,7 +105,8 @@ XrdScheduler *XrdHttpProtocol::Sched = 0; // System scheduler
 XrdBuffManager *XrdHttpProtocol::BPool = 0; // Buffer manager
 XrdSysError XrdHttpProtocol::eDest = 0; // Error message handler
 XrdSecService *XrdHttpProtocol::CIA = 0; // Authentication Server
-
+int XrdHttpProtocol::m_bio_type = 0; // BIO type identifier for our custom BIO.
+BIO_METHOD *XrdHttpProtocol::m_bio_method = NULL; // BIO method constructor.
 
 /******************************************************************************/
 /*            P r o t o c o l   M a n a g e m e n t   S t a c k s             */
@@ -171,6 +173,41 @@ extern "C" {
   }
 }
 
+
+
+
+/******************************************************************************/
+/*               U g l y  O p e n S S L   w o r k a r o u n d s               */
+/******************************************************************************/
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+void *BIO_get_data(BIO *bio) {
+  return bio->ptr;
+}
+void BIO_set_data(BIO *bio, void *ptr) {
+  bio->ptr = ptr;
+}
+#if OPENSSL_VERSION_NUMBER < 0x1000105fL
+int BIO_get_flags(BIO *bio) {
+  return bio->flags;
+}
+#endif
+void BIO_set_flags(BIO *bio, int flags) {
+  bio->flags = flags;
+}
+int BIO_get_init(BIO *bio) {
+  return bio->init;
+}
+void BIO_set_init(BIO *bio, int init) {
+  bio->init = init;
+}
+void BIO_set_shutdown(BIO *bio, int shut) {
+  bio->shutdown = shut;
+}
+int BIO_get_shutdown(BIO *bio) {
+  return bio->shutdown;
+}
+    
+#endif
 /******************************************************************************/
 /*               X r d H T T P P r o t o c o l   C l a s s                    */
 /******************************************************************************/
@@ -407,8 +444,159 @@ char *XrdHttpProtocol::GetClientIPStr() {
 }
 
 
+// Various routines for handling XrdLink as BIO objects within OpenSSL.
+#if OPENSSL_VERSION_NUMBER < 0x1000105fL
+int BIO_XrdLink_write(BIO *bio, const char *data, size_t datal, size_t *written)
+{
+  if (!data || !bio) {
+    *written = 0;
+    return 0;
+  }
+  
+  XrdLink *lp=static_cast<XrdLink *>(BIO_get_data(bio));
+  
+  errno = 0;
+  int ret = lp->Send(data, datal);
+  BIO_clear_retry_flags(bio);
+  if (ret <= 0) {
+    *written = 0;
+    if ((ret == EINTR) || (ret == EINPROGRESS) || (ret == EAGAIN) || (ret == EWOULDBLOCK))
+      BIO_set_retry_read(bio);
+    return ret;
+  }
+  *written = ret;
+  return 1;
+}
+#else
+int BIO_XrdLink_write(BIO *bio, const char *data, int datal)
+{
+  if (!data || !bio) {
+    errno = ENOMEM;
+    return -1;
+  }
+
+  errno = 0;
+  XrdLink *lp = static_cast<XrdLink *>(BIO_get_data(bio));
+  int ret = lp->Send(data, datal);
+  BIO_clear_retry_flags(bio);
+  if (ret <= 0) {
+    if ((ret == EINTR) || (ret == EINPROGRESS) || (ret == EAGAIN) || (ret == EWOULDBLOCK))
+      BIO_set_retry_read(bio);
+  }
+  return ret;
+}
+#endif
 
 
+#if OPENSSL_VERSION_NUMBER < 0x1000105fL
+static int BIO_XrdLink_read(BIO *bio, char *data, size_t datal, size_t *read)
+{
+  if (!data || !bio) {
+    *read = 0;
+    return 0;
+  }
+
+  errno = 0;
+  
+  XrdLink *lp = static_cast<XrdLink *>(BIO_get_data(bio));  
+  int ret = lp->Recv(data, datal);
+  BIO_clear_retry_flags(bio);
+  if (ret <= 0) {
+    *read = 0;
+    if ((ret == EINTR) || (ret == EINPROGRESS) || (ret == EAGAIN) || (ret == EWOULDBLOCK))
+      BIO_set_retry_read(bio);
+    return ret;
+  }
+  *read = ret;
+}
+#else
+static int BIO_XrdLink_read(BIO *bio, char *data, int datal)
+{
+  if (!data || !bio) {
+    errno = ENOMEM;
+    return -1;
+  }
+
+  errno = 0;
+  XrdLink *lp = static_cast<XrdLink *>(BIO_get_data(bio));
+  int ret = lp->Recv(data, datal);
+  BIO_clear_retry_flags(bio);
+  if (ret <= 0) {
+    if ((ret == EINTR) || (ret == EINPROGRESS) || (ret == EAGAIN) || (ret == EWOULDBLOCK))
+      BIO_set_retry_read(bio);
+  }
+  return ret;
+}
+#endif
+
+
+static int BIO_XrdLink_create(BIO *bio)
+{
+    
+    
+  BIO_set_init(bio, 0);
+  //BIO_set_next(bio, 0);
+  BIO_set_data(bio, NULL);
+  BIO_set_flags(bio, 0);
+  
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+
+  bio->num = 0;
+
+#endif
+  
+  return 1;
+}
+
+
+static int BIO_XrdLink_destroy(BIO *bio)
+{
+  if (bio == NULL) return 0;
+  if (BIO_get_shutdown(bio)) {
+    if (BIO_get_data(bio)) {
+      static_cast<XrdLink*>(BIO_get_data(bio))->Close();
+    }
+    BIO_set_init(bio, 0);
+    BIO_set_flags(bio, 0);
+  }
+  return 1;
+}
+
+
+static long BIO_XrdLink_ctrl(BIO *bio, int cmd, long num, void * ptr)
+{
+  long ret = 1;
+  switch (cmd) {
+  case BIO_CTRL_GET_CLOSE:
+    ret = BIO_get_shutdown(bio);
+    break;
+  case BIO_CTRL_SET_CLOSE:
+    BIO_set_shutdown(bio, (int)num);
+    break;  
+  case BIO_CTRL_DUP:
+  case BIO_CTRL_FLUSH:
+    ret = 1;
+    break;
+  default:
+    ret = 0;
+    break;
+  }
+  return ret;
+}
+
+
+BIO *XrdHttpProtocol::CreateBIO(XrdLink *lp)
+{
+  if (m_bio_method == NULL)
+    return NULL;
+
+  BIO *ret = BIO_new(m_bio_method);
+
+  BIO_set_shutdown(ret, 0);
+  BIO_set_data(ret, lp);
+  BIO_set_init(ret, 1);
+  return ret;
+}
 
 
 /******************************************************************************/
@@ -444,7 +632,7 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
   if (ishttps && !ssldone) {
 
       if (!ssl) {
-          sbio = BIO_new_socket(Link->FDnum(), BIO_NOCLOSE);
+          sbio = CreateBIO(Link);
           BIO_set_nbio(sbio, 1);
           ssl = SSL_new(sslctx);
         }
@@ -545,9 +733,12 @@ int XrdHttpProtocol::Process(XrdLink *lp) // We ignore the argument here
       }
 
 
-      if (CurrentReq.request == CurrentReq.rtUnknown) {
+      if (CurrentReq.request == CurrentReq.rtUnset) {
         TRACE(DEBUG, " Parsing first line: " << tmpline);
-        CurrentReq.parseFirstLine((char *)tmpline.c_str(), rc);
+        int result = CurrentReq.parseFirstLine((char *)tmpline.c_str(), rc);
+        if (result < 0) {
+          TRACE(DEBUG, " Parsing of first line failed with " << result);
+        }
       }
       else
         CurrentReq.parseLine((char *)tmpline.c_str(), rc);
@@ -838,6 +1029,41 @@ int XrdHttpProtocol::Config(const char *ConfigFN, XrdOucEnv *myEnv) {
   char *var;
   int cfgFD, GoNo, NoGo = 0, ismine;
 
+  // Initialize our custom BIO type.
+  if (!m_bio_type) {
+
+    #if OPENSSL_VERSION_NUMBER < 0x10100000L
+      m_bio_type = (26|0x0400|0x0100);
+      m_bio_method = static_cast<BIO_METHOD*>(OPENSSL_malloc(sizeof(BIO_METHOD)));
+ 
+      if (m_bio_method) {
+        memset(m_bio_method, '\0', sizeof(BIO_METHOD));
+        m_bio_method->type = m_bio_type;
+        m_bio_method->bwrite = BIO_XrdLink_write;
+        m_bio_method->bread = BIO_XrdLink_read;
+        m_bio_method->create = BIO_XrdLink_create;
+        m_bio_method->destroy = BIO_XrdLink_destroy;
+        m_bio_method->ctrl = BIO_XrdLink_ctrl;
+      }
+    #else
+      // OpenSSL 1.1 has an internal counter for generating unique types.
+      // We'll switch to that when widely available.
+      m_bio_type = BIO_get_new_index();
+      m_bio_method = BIO_meth_new(m_bio_type, "xrdhttp-bio-method");
+      
+      if (m_bio_method) {
+        BIO_meth_set_write(m_bio_method, BIO_XrdLink_write);
+        BIO_meth_set_read(m_bio_method, BIO_XrdLink_read);
+        BIO_meth_set_create(m_bio_method, BIO_XrdLink_create);
+        BIO_meth_set_destroy(m_bio_method, BIO_XrdLink_destroy);
+        BIO_meth_set_ctrl(m_bio_method, BIO_XrdLink_ctrl);
+      }
+      
+    #endif
+    
+
+  }
+
   // Open and attach the config file
   //
   if ((cfgFD = open(ConfigFN, O_RDONLY, 0)) < 0)
@@ -1167,11 +1393,10 @@ int XrdHttpProtocol::BuffgetData(int blen, char **data, bool wait) {
 
 /// Send some data to the client
 
-int XrdHttpProtocol::SendData(char *body, int bodylen) {
+int XrdHttpProtocol::SendData(const char *body, int bodylen) {
+
   int r;
 
-
-
   if (body && bodylen) {
     TRACE(REQ, "Sending " << bodylen << " bytes");
     if (ishttps) {
@@ -1190,75 +1415,92 @@ int XrdHttpProtocol::SendData(char *body, int bodylen) {
   return 0;
 }
 
+int XrdHttpProtocol::StartSimpleResp(int code, const char *desc, const char *header_to_add, long long bodylen) {
+  std::stringstream ss;
+  const std::string crlf = "\r\n";
+
+  ss << "HTTP/1.1 " << code << " ";
+  if (desc) {
+    ss << desc;
+  } else {
+    if (code == 200) ss << "OK";
+    else if (code == 206) ss << "Partial content";
+    else if (code == 302) ss << "Redirect";
+    else if (code == 404) ss << "Not found";
+    else ss << "Unknown";
+  }
+  ss << crlf;
+
+  if (bodylen >= 0) ss << "Content-Length: " << bodylen << crlf;
+
+  if (header_to_add)
+    ss << header_to_add << crlf;
+
+  ss << crlf;
+
+  const std::string &outhdr = ss.str();
+  TRACEI(RSP, "Sending resp: " << code << " header len:" << outhdr.size());
+  if (SendData(outhdr.c_str(), outhdr.size()))
+    return -1;
+
+  return 0;
+}
+
+int XrdHttpProtocol::StartChunkedResp(int code, const char *desc, const char *header_to_add) {
+  const std::string crlf = "\r\n";
+
+  std::stringstream ss;
+  if (header_to_add) {
+    ss << header_to_add << crlf;
+  }
+  ss << "Transfer-Encoding: chunked";
+
+  TRACEI(RSP, "Starting chunked response");
+  return StartSimpleResp(code, desc, ss.str().c_str(), -1);
+}
+
+int XrdHttpProtocol::ChunkResp(const char *body, long long bodylen) {
+  const std::string crlf = "\r\n";
+  long long chunk_length = bodylen;
+  if (bodylen <= 0) {
+    chunk_length = body ? strlen(body) : 0;
+  }
+  std::stringstream ss;
+
+  ss << std::hex << chunk_length << std::dec << crlf;
+
+  const std::string &chunkhdr = ss.str();
+  TRACEI(RSP, "Sending encoded chunk of size " << chunk_length);
+  if (SendData(chunkhdr.c_str(), chunkhdr.size()))
+    return -1;
+
+  if (body && SendData(body, chunk_length))
+    return -1;
+
+  return (SendData(crlf.c_str(), crlf.size())) ? -1 : 0;
+}
+
 /// Sends a basic response. If the length is < 0 then it is calculated internally
 /// Header_to_add is a set of header lines each CRLF terminated to be added to the header
 /// Returns 0 if OK
 
-int XrdHttpProtocol::SendSimpleResp(int code, char *desc, char *header_to_add, char *body, long long bodylen) {
-  char outhdr[1024];
-  char b[32];
-  long long l;
-  const char *crlf = "\r\n";
-  outhdr[0] = '\0';
+int XrdHttpProtocol::SendSimpleResp(int code, const char *desc, const char *header_to_add, const char *body, long long bodylen) {
 
-
-  //
-  // Prepare the header
-  //
-  strcat(outhdr, "HTTP/1.1 ");
-  sprintf(b, "%d ", code);
-  strcat(outhdr, b);
-
-  if (desc) strcat(outhdr, desc);
-  else {
-    if (code == 200) strcat(outhdr, "OK");
-    else if (code == 206) strcat(outhdr, "Partial content");
-    else if (code == 302) strcat(outhdr, "Redirect");
-    else if (code == 404) strcat(outhdr, "Not found");
-    else strcat(outhdr, "Unknown");
-  }
-  strncat(outhdr, crlf, 2);
-
-  //strcat(outhdr, "Content-Type: text/html");
-  //strncat(outhdr, crlf, 2);
-
-  l = bodylen;
-  if (l <= 0) {
-    if (body) l = strlen(body);
-    else l = 0;
+  long long content_length = bodylen;
+  if (bodylen <= 0) {
+    content_length = body ? strlen(body) : 0;
   }
 
-  sprintf(b, "%lld", l);
-  strcat(outhdr, "Content-Length: ");
-  strcat(outhdr, b);
-  strncat(outhdr, crlf, 2);
-
-  if (header_to_add) {
-    strcat(outhdr, header_to_add);
-    strncat(outhdr, crlf, 2);
-  }
-  strncat(outhdr, crlf, 2);
-
-  unsigned int hdrlen = strlen(outhdr);
-  if (hdrlen >= sizeof(outhdr))
-    TRACEI(ALL, "WARNING: header size too large!");
-  
-  //
-  // Send the header
-  //
-  TRACEI(RSP, "Sending resp: " << code << " len:" << l);
-
-  if (SendData(outhdr, hdrlen))
+  if (StartSimpleResp(code, desc, header_to_add, content_length) < 0)
     return -1;
 
   //
   // Send the data
   //
   if (body)
-    return SendData(body, l);
+    return SendData(body, content_length);
 
   return 0;
-
 }
 
 int XrdHttpProtocol::Configure(char *parms, XrdProtocol_Config * pi) {
@@ -1546,7 +1788,7 @@ void XrdHttpProtocol::Cleanup() {
   if (SecEntity.host) free(SecEntity.host);
   if (SecEntity.moninfo) free(SecEntity.moninfo);
 
-  memset(&SecEntity, 0, sizeof (SecEntity));
+  SecEntity.Reset();
 
   if (Addr_str) free(Addr_str);
   Addr_str = 0;
@@ -1582,7 +1824,7 @@ void XrdHttpProtocol::Reset() {
   //  cumWrites = 0;
   //  totReadP = 0;
 
-  memset(&SecEntity, 0, sizeof (SecEntity));
+  SecEntity.Reset();
   SecEntity.tident = XrdHttpSecEntityTident;
   ishttps = false;
   ssldone = false;
