@@ -302,6 +302,7 @@ int XrdHttpReq::parseRWOp(char *str) {
       len_ok += len;
       rwOps_split.push_back(nfo);
     }
+    length += len_ok;
 
 
   }
@@ -814,7 +815,13 @@ int XrdHttpReq::ProcessHTTPReq() {
   //
 
   switch (request) {
+    case XrdHttpReq::rtUnset:
     case XrdHttpReq::rtUnknown:
+    {
+      prot->SendSimpleResp(400, NULL, NULL, (char *) "Request unknown", 0);
+      reset();
+      return -1;
+    }
     case XrdHttpReq::rtMalformed:
     {
       prot->SendSimpleResp(400, NULL, NULL, (char *) "Request malformed", 0);
@@ -978,7 +985,7 @@ int XrdHttpReq::ProcessHTTPReq() {
         {
 
           if ( ((reqstate == 3) && (rwOps.size() > 1)) ||
-            (writtenbytes >= filesize) ) {
+            (writtenbytes >= length) ) {
 
             // Close() if this was a readv or we have finished, otherwise read the next chunk
 
@@ -1090,7 +1097,7 @@ int XrdHttpReq::ProcessHTTPReq() {
         l = resourceplusopaque.length() + 1;
         xrdreq.open.dlen = htonl(l);
         xrdreq.open.mode = htons(kXR_ur | kXR_uw | kXR_gw | kXR_gr | kXR_or);
-        xrdreq.open.options = htons(kXR_mkpath | kXR_open_wrto );
+        xrdreq.open.options = htons(kXR_mkpath | kXR_open_wrto | kXR_delete);
 
         if (!prot->Bridge->Run((char *) &xrdreq, (char *) resourceplusopaque.c_str(), l)) {
           prot->SendSimpleResp(404, NULL, NULL, (char *) "Could not run request.", 0);
@@ -1671,6 +1678,12 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                        &filesize,
                        &fileflags,
                        &filemodtime);
+
+                // We will default the response size specified by the headers; if that
+                // wasn't given, use the file size.
+                if (!length) {
+                    length = filesize;
+                }
               }
               else
                 TRACEI(REQ, "Can't find the stat information for '" << resource << "' Internal error?");
@@ -1710,6 +1723,12 @@ int XrdHttpReq::PostProcessHTTPReq(bool final_) {
                         &filesize,
                         &fileflags,
                         &filemodtime);
+
+                  // As above: if the client specified a response size, we use that.
+                  // Otherwise, utilize the filesize
+                  if (!length) {
+                    length = filesize;
+                  }
                 }
                 else
                   TRACEI(ALL, "GET returned no STAT information. Internal error?");
@@ -2254,7 +2273,7 @@ void XrdHttpReq::reset() {
   if (ralist) free(ralist);
   ralist = 0;
 
-  request = rtUnknown;
+  request = rtUnset;
   resource = "";
   allheaders.clear();
 
